#!/bin/sh

MAGIC_LABEL="pstorage-hotplug"
MAGIC_FILE=".pstorage.hotplug"

source /usr/libexec/pstorage/service-lib.sh

mon_remove_dirty_hook() {
	true
}

logmsg() {
	local datetime=$(fmt_time)
	echo "$datetime" "$@"
}

check_config() {
	[ "$ENABLE_HOTPLUG" != "yes" ] && exit
	[ -z "$HOTPLUG_CLUSTER" ] && exit
	mkdir -p "$CLUSTERS_DIR/$HOTPLUG_CLUSTER"
	LIST_FILE="$CLUSTERS_DIR/$HOTPLUG_CLUSTER/cs.list"
	LOCK_FILE="$CLUSTERS_DIR/$HOTPLUG_CLUSTER/.lock"
	[ -z "$HOTPLUG_TIER" ] && HOTPLUG_TIER="0"
	[ -z "$HOTPLUG_LOG_FILE" ] && HOTPLUG_LOG_FILE="/var/log/pstorage/hotplug.log"
}

get_disk_path() {
	[ "$DEVPATH" ] && return
	DEVPATH=$(echo "$1" | sed 's:^/dev:/block:')
}

get_part_label() {
	[ "$ID_FS_LABEL" ] && return
	ID_FS_LABEL=$(e2label "$1" 2>/dev/null)
}

adddisk()
{
	get_disk_path "$DEVNAME"
	logmsg "New disk found: $DEVNAME"
	SYSFS=$(mount -l -t sysfs | cut -d' ' -f3)
	if [ -z "$SYSFS" ]; then
		logmsg "Sysfs not found, exit"
		return
	fi
	LANG=C /sbin/parted "$DEVNAME" print
	PARTNUM=$(ls "$SYSFS$DEVPATH"/*/partition 2>/dev/null | wc -l)
	HOLDERS=$(ls "$SYSFS$DEVPATH"/holders/ 2>/dev/null)
	ROTATIONAL=$(cat "$SYSFS$DEVPATH"/queue/rotational)
	if [ "$ROTATIONAL" -ne 1 ]; then
		logmsg "$DEVNAME: Disk is not rotational, ignore"
		return
	fi
	if [ "$HOLDERS" ]; then
		logmsg "$DEVNAME: Disk is already in use, ignore"
		return
	fi
	logmsg "$DEVNAME: Num of partitions: $PARTNUM"
	[ "$PARTNUM" -ne 0 ] && return
	/usr/libexec/pstorage/prepare_pstorage_drive "$DEVNAME" -y
}

addpartition()
{
	logmsg "New partition found: $DEVNAME"
	get_part_label "$DEVNAME"
	if [ "$ID_FS_LABEL" != "$MAGIC_LABEL" ]; then
		logmsg "$DEVNAME: Invalid label '$ID_FS_LABEL', ignore"
		return
	fi
	logmsg "$DEVNAME: Detected right label"
	MOUNTPOINT=$(grep "$DEVNAME" /proc/self/mounts | cut -d' ' -f2)
	if [ "$MOUNTPOINT" ]; then
		logmsg "$DEVNAME: Already mounted: $MOUNTPOINT"
		return
	fi
	mkdir -p "$HOTPLUG_MOUNT_PATH"
	MOUNTPOINT=$(mktemp -d "$HOTPLUG_MOUNT_PATH"/pstorage-XXXXXXXX)
	logmsg "$DEVNAME: Trying to mount at $MOUNTPOINT"
	mount "$DEVNAME" "$MOUNTPOINT" || return
	touch "$MOUNTPOINT"/.pstorage.hotplug

	if [ -d "$MOUNTPOINT/cs" ]; then
		# check that the CS cluster name equals $HOTPLUG_CLUSTER
		grep '^\s*'"$HOTPLUG_CLUSTER"'\s*$' "$MOUNTPOINT/cs/control/cluster_name" >/dev/null 2>&1 || return
		# check that the CS doesn't have journal (we don't support SSDs yet)
		if [ -e "$MOUNTPOINT/cs/control/journal" ]; then
			logmsg "$DEVNAME: CS has a journal, ignore"
			return
		fi
		# lock per-cluster lock before accessing list
		( flock -x -w 10 100 || exit 1
			if [ -f "$LIST_FILE" ]; then
				if ! grep "$MOUNTPOINT/cs" "$LIST_FILE" >/dev/null 2>&1; then
					echo "$MOUNTPOINT/cs" >> "$LIST_FILE"
				fi
			else
				echo "# autogenerated, unique" > "$LIST_FILE"
				echo "$MOUNTPOINT/cs" >> "$LIST_FILE"
			fi
		) 100>"$LOCK_FILE" || logmsg "$DEVNAME: Failed to obtain lock: $LOCK_FILE"
	else
		/usr/bin/pstorage -c "$HOTPLUG_CLUSTER" make-cs -r "$MOUNTPOINT/cs" -t "$HOTPLUG_TIER" || return
	fi
	/etc/init.d/pstorage-csd start "$MOUNTPOINT/cs"
}

removepartition() {
	logmsg "Partition removed: $DEVNAME"
	if [ -b "$DEVNAME" ]; then
		get_part_label "$DEVNAME"
		[ "$ID_FS_LABEL" != "$MAGIC_LABEL" ] && return
		logmsg "$DEVNAME: Detected right label"
	fi
	MOUNTPOINT=$(grep "$DEVNAME" /proc/self/mounts | cut -d' ' -f2 | grep '^'"$HOTPLUG_MOUNT_PATH"'/pstorage-' | head -n 1)
	[ -z "$MOUNTPOINT" ] && return
	logmsg "$DEVNAME: Detected mountpoint: $MOUNTPOINT"
	# lock per-cluster lock before accessing list
	( flock -x -w 10 100 || exit 1
		grep "$MOUNTPOINT" "$LIST_FILE" | while read cs; do
			PID_DIR=$(get_pid_dir "$cs")
			echo -n "after disk unplug" > "$PID_DIR/stop-reason"
			stop_monitor "$HOTPLUG_CLUSTER" "$cs"
		done
		grep -v "^$MOUNTPOINT" "$LIST_FILE" > "$LIST_FILE".tmp
		mv "$LIST_FILE".tmp "$LIST_FILE"
	) 100>"$LOCK_FILE" || logmsg "$DEVNAME: Failed to obtain lock: $LOCK_FILE"
	umount "$MOUNTPOINT"
	rm -rf "$MOUNTPOINT"
}

if ! [ "$ACTION" -a "$DEVTYPE" -a "$DEVNAME" ]; then
	echo "Not enough arguments."
	exit 1
fi

HANDLER="$ACTION$DEVTYPE"
case "$HANDLER" in
	adddisk|addpartition|removepartition)
		check_config
		"$HANDLER" >> "$HOTPLUG_LOG_FILE" 2>&1
		;;
esac
